#!/bin/bash
# set -x

# Catch the SIGTERM signal. See if we're exiting because of a reboot or a shutdown.
function term_trap {
    REBOOT=$(systemctl list-jobs | grep -E -q 'reboot.target.*start' && echo "rebooting" || echo "not_rebooting")
    PWROFF=$(systemctl list-jobs | grep -E -q 'poweroff.target.*start' && echo "poweroff" || echo "not_poweroff")
    /usr/bin/logger --tag "csrv.sh" "(term_trap) $REBOOT $PWROFF"
}

# Determine if the camera is integrated or not.
isIntegrated() {
    NODE_NAME=$(lspci -s "03:00.0")
    if [ -z "$NODE_NAME" ]; then
        false
    else
        true
    fi
}


# Remove the NVME drive(s) from the system.
function remove_drives {
    # Make sure everything is sync'd.
    sync; sync

    # Find any "/dev" entries.
    find /dev | grep -E 'nvme[0-9]n[0-9]$' | while read nvme_dev; do
    	# Find any/all partitions and make sure they are unmounted.
        for part in {0..9}; do
            if [ -e "$nvme_dev"p"$part" ]; then
                /usr/bin/logger --tag "csrv.sh" "(remove_drives) Unmount $nvme_dev"p"$part"
                umount -q "$nvme_dev"p"$part"
            fi
        done

        # Flush the block devices, just to be certain.
    	/usr/bin/logger --tag "csrv.sh" "(remove_drives) Flushing $nvme_dev"
        /usr/bin/blockdev --flushbufs $nvme_dev
    done

    # Find all the NVME drives.
    find /sys/devices -type d | grep -E '/nvme$' | while read nvme_item; do
    	# Substitute the nvme folder name with the remove control item.
        remove_path=${nvme_item/nvme/remove}

        # Issue the remove command.
        /usr/bin/logger --tag "csrv.sh" "(remove_drives) $remove_path"
        echo 1 > $remove_path
    done
}

# Make sure we have a link for the WSS.
# if [ ! -L "/tmp/www" ]; then
#     ln -s /home/swdev/sandbox/web /tmp/www
# fi

# terminator -x tail -f /data/cs.log &

# Setup the environment with values we want/need.
export USER=fascam
export SUDO_USER=fascam
export HOME=/home/$USER
export SCRATCH=$HOME/.config/Fastec/scratch

# Before we can load our device drivers, we need to disable ACS on the
# two ports used for P2P DMA transfers on the camera head's switch.
SWITCH_VID="10b5"
SWITCH_DID="8714"
PCI_ACS_REG="f2a"
SWITCH_BUS=$(lspci -d $SWITCH_VID:$SWITCH_DID | cut -d':' -f1 | tail -1)
setpci -s 0000:$SWITCH_BUS:01.0 $PCI_ACS_REG.w=0000
setpci -s 0000:$SWITCH_BUS:02.0 $PCI_ACS_REG.w=0000

# Make sure we can find the libraries.
ldconfig /usr/local/lib

# Remove any "default" Altera drivers.
# Better would be to blacklist the drivers. Refer to https://wiki.archlinux.org/index.php/Kernel_modules#Blacklisting.
rmmod altera_cvp &> /dev/null
rmmod fpga_mgr &> /dev/null

# Remove any old drivers.
rmmod fi_fpga &> /dev/null
rmmod fi_pci &> /dev/null

# Add in our new drivers.
modprobe fi_pci bar_select=2
modprobe fi_fpga

# Log the link status, if the script exists.
if [ -f /usr/local/bin/fastec/utilities/pci_linkstatus.sh ]; then
    # Append the link status information.
    /usr/local/bin/fastec/utilities/pci_linkstatus.sh >> $SCRATCH/pci_linkstatus.log

    # Check to see if we need to reduce the file size.
    if isIntegrated; then
        LINES_PER_LOG=7
    else
        LINES_PER_LOG=9
    fi

    MAX_LINES_PER_LOG=$(( $LINES_PER_LOG*100000 ))
    CLIP_LINES_PER_LOG=$(( $LINES_PER_LOG*90000 ))
    
    # How many lines are in the file.
    lines_in_file=$(wc -l < "$SCRATCH/pci_linkstatus.log")
    /usr/bin/logger --tag "csrv.sh" "MAX_LINES_PER_LOG=$MAX_LINES_PER_LOG, lines_in_file=$lines_in_file"
    
    # Are we larger than our maximum?
    if (( lines_in_file > MAX_LINES_PER_LOG )); then
        tail --lines=$CLIP_LINES_PER_LOG $SCRATCH/pci_linkstatus.log > $SCRATCH/pci_linkstatus.log.clip && mv $SCRATCH/pci_linkstatus.log.clip $SCRATCH/pci_linkstatus.log
    fi
fi

trap term_trap SIGTERM

while true; do
    # Start up our web socket server after a small delay to let the 
    # camera server to start.
    # ( sleep 2; /usr/local/bin/fi_wss &> $SCRATCH/wss.log ) &

    # Start up our camera server.
    /usr/local/bin/camera_server &> $SCRATCH/cs.log 
    exit_loop=$?
    /usr/bin/logger --tag "csrv.sh" "Camera Server exit status: $exit_loop"
    /usr/bin/logger --tag "csrv.sh" "Camera Server: $REBOOT $PWROFF"

    # Are we in the middle of a system shutdown?
    if [ "$PWROFF" = "poweroff" ]; then
        # Remove the device
        /usr/bin/logger --tag "csrv.sh" "Remove the NVME drive" 
        remove_drives

        # Tell the FPGA to turn off the camera head.
        /usr/bin/logger --tag "csrv.sh" "Power off the head now" 
        echo "w 0x1BC 0x3210ABCD" > /proc/fi_pci/regs
    fi

    # The exit status of 115 ('s') means to shutdown.
    if [ "$exit_loop" = "115" ]; then
        # Remove the device
        /usr/bin/logger --tag "csrv.sh" "Remove the NVME drive" 
        remove_drives

        # Tell the FPGA to turn off the camera head.
        /usr/bin/logger --tag "csrv.sh" "Power off the head now" 
        echo "w 0x1BC 0x3210ABCD" > /proc/fi_pci/regs
        sleep 1

        # Issue a shutdown request.
        /usr/bin/logger --tag "csrv.sh" "Shutdown now"
        /usr/bin/systemctl poweroff
    fi

    # An exit status of 82 ('R') measn to reboot
    if [ "$exit_loop" = "82" ]; then
        # Make sure things are settled.
        sync; sync

        # Request a reboot.
        /usr/bin/logger --tag "csrv.sh" "Reboot now"
        /usr/bin/systemctl reboot
    fi

    # Kill any WSS instances.
    killall -q fi_wss

    # Is the camera server asking for a restart?
    # The exit status of 114 ('r') means to restart. 
    if [ "$exit_loop" != "114" ]; then
        exit 0
    fi
done
